@@ -11,9 +11,10 @@ db = SQLAlchemy()
 
 
 from .util import DB_STATS
+from .util.request import AcceptRequest
 
 
-__version__ = '0.5.4'
+__version__ = '0.6.0'
 
 
 requests_session = requests.Session()
@@ -33,6 +34,7 @@ from .auth import models as auth_models
 
 def create_app(config=None, **kwargs):
     app = Flask('evesrp', **kwargs)
+    app.request_class = AcceptRequest
     app.config.from_object('evesrp.default_config')
     if config is not None:
         app.config.from_pyfile(config)
@@ -61,9 +63,9 @@ def create_app(config=None, **kwargs):
     app.add_url_rule(rule='/', view_func=index)
     for error_code in (400, 403, 404, 500):
         app.register_error_handler(error_code, error_page)
-    app.register_blueprint(divisions.blueprint, url_prefix='/divisions')
+    app.register_blueprint(divisions.blueprint, url_prefix='/division')
     app.register_blueprint(login.blueprint)
-    app.register_blueprint(requests.blueprint)
+    app.register_blueprint(requests.blueprint, url_prefix='/request')
     app.register_blueprint(api.api, url_prefix='/api')
     app.register_blueprint(api.filters, url_prefix='/api/filter')
 
@@ -107,16 +109,22 @@ def _copy_config_to_authmethods():
 
 # Request detail URL setup
 def _copy_url_converter_config():
-    current_app.ship_urls = {}
-    ship_transformers = current_app.config.get('SRP_SHIP_URL_TRANSFORMERS')
-    if ship_transformers is not None:
-        for transformer in ship_transformers:
-            current_app.ship_urls[transformer.name] = transformer
-    current_app.pilot_urls = {}
-    pilot_transformers = current_app.config.get('SRP_PILOT_URL_TRANSFORMERS')
-    if pilot_transformers is not None:
-        for transformer in pilot_transformers:
-            current_app.pilot_urls[transformer.name] = transformer
+    url_transformers = {}
+    for config_key, config_value in current_app.config.items():
+        if config_value is None:
+            continue
+        index = config_key.rfind('_URL_TRANSFORMERS')
+        if not config_key.endswith('_URL_TRANSFORMERS')\
+                or not config_key.startswith('SRP_'):
+            continue
+        attribute = config_key.replace('SRP_', '', 1)
+        attribute = attribute.replace('_URL_TRANSFORMERS', '', 1)
+        attribute = attribute.lower()
+        url_transformers[attribute] = {t.name:t for t in config_value}
+    current_app.url_transformers = url_transformers
+    # temporary compatibility
+    current_app.ship_urls = url_transformers.get('ship_type', None)
+    current_app.pilot_urls = url_transformers.get('pilot', None)
 
 
 # Requests session setup
