@@ -1,3 +1,4 @@
+from itertools import groupby
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.ext.hybrid import hybrid_property
@@ -152,6 +153,24 @@ class User(Entity):
         """Part of the interface for Flask-Login."""
         return str(self.id)
 
+    def submit_divisions(self):
+        """Get a list of the divisions this user is able to submit requests to.
+
+        :returns: A list of tuples. The tuples are in the form (division.id,
+            division.name)
+        :rtype: list
+        """
+        submit_perms = self.permissions\
+                .filter_by(permission=PermissionType.submit)\
+                .subquery()
+        divisions = db.session.query(Division).join(submit_perms)\
+                .order_by(Division.name)
+        # Remove duplicates and sort divisions by name
+        choices = []
+        for name, group in groupby(divisions, lambda d: d.name):
+            choices.append((next(group).id, name))
+        return choices
+
 
 class Note(db.Model, AutoID, Timestamped, AutoName):
     user_id = db.Column(db.Integer, db.ForeignKey('user.id'), nullable=False)
@@ -200,6 +219,9 @@ class Pilot(db.Model, AutoID, AutoName):
         return "{x.__class__.__name__}({x.user}, '{x.name}', {x.id})".format(
                 x=self)
 
+    def __str__(self):
+        return self.name
+
 
 class Group(Entity):
     """Base class for a group of users.
@@ -251,6 +273,38 @@ class Permission(db.Model, AutoID, AutoName):
                "{x.division})").format(x=self)
 
 
+class TransformerRef(db.Model, AutoID, AutoName):
+    """Stores associations between :py:class:`~.Transformer`\s and
+    :py:class:`.Division`\s.
+    """
+
+    __table_args__ = (
+        db.UniqueConstraint('division_id', 'attribute_name'),
+    )
+
+    #: The attribute this transformer is applied to.
+    attribute_name = db.Column(db.String(50), nullable=False)
+
+    #: The transformer instance.
+    transformer = db.Column(db.PickleType, nullable=True)
+
+    division_id = db.Column(db.Integer, db.ForeignKey('division.id'),
+            nullable=False)
+
+    #: The division the transformer is associated with
+    division = db.relationship('Division',
+            back_populates='division_transformers')
+
+    @db.validates('transformer')
+    def prune_null_transformers(self, attr, transformer):
+        """Removes :py:class:`TransformerRef`\s when :py:attr:`.transformer` is
+        removed.
+        """
+        if transformer is None:
+            self.division = None
+        return transformer
+
+
 class Division(db.Model, AutoID, AutoName):
     """A reimbursement division.
 
@@ -268,9 +322,17 @@ class Division(db.Model, AutoID, AutoName):
     #: :py:class:`Request` s filed under this division.
     requests = db.relationship(Request, back_populates='division')
 
-    ship_transformer = db.Column(db.PickleType, nullable=True, default=None)
+    division_transformers = db.relationship(TransformerRef,
+            collection_class=attribute_mapped_collection('attribute_name'),
+            back_populates='division', cascade='delete,delete-orphan')
 
-    pilot_transformer = db.Column(db.PickleType, nullable=True, default=None)
+    #: A mapping of attribute names to :py:class:`~.transformer.Transformer`
+    #: instances.
+    transformers = association_proxy(
+            'division_transformers',
+            'transformer',
+            creator=lambda attr, trans:
+                    TransformerRef(attribute_name=attr, transformer=trans))
 
     @property
     def permissions(self):
