@@ -1,9 +1,8 @@
 from collections import OrderedDict
-from itertools import groupby
 import re
 
 from flask import render_template, abort, url_for, flash, Markup, request,\
-    redirect, current_app, Blueprint, Markup
+    redirect, current_app, Blueprint, Markup, json, jsonify
 from flask.views import View
 from flask.ext.login import login_required, current_user
 from flask.ext.wtf import Form
@@ -46,14 +45,16 @@ class RequestListing(View):
         """
         raise NotImplementedError()
 
-    def dispatch_request(self, division_id=None, page=1):
+    def dispatch_request(self, division_id=None, page=1, **kwargs):
         """Returns the response to requests.
 
         Part of the :py:class:`flask.views.View` interface.
         """
+        if request.wants_json or request.is_xhr:
+            return jsonify(requests=self.requests(division_id))
         pager = self.requests(division_id).paginate(page, per_page=20)
         return render_template(self.template,
-                pager=pager)
+                pager=pager, **kwargs)
 
     @property
     def _load_options(self):
@@ -106,12 +107,12 @@ class PermissionRequestListing(RequestListing):
         self.permissions = permissions
         self.statuses = statuses
 
-    def dispatch_request(self, division_id=None, page=1):
+    def dispatch_request(self, division_id=None, page=1, **kwargs):
         if not current_user.has_permission(self.permissions):
             abort(403)
         else:
             return super(PermissionRequestListing, self).dispatch_request(
-                    division_id, page)
+                    division_id, page, **kwargs)
 
     def requests(self, division_id=None):
         user_perms = db.session.query(Permission.id.label('permission_id'),
@@ -152,8 +153,8 @@ class PayoutListing(PermissionRequestListing):
         """
         if not current_user.has_permission(self.permissions):
             abort(403)
-        pager = self.requests(division_id).paginate(page, per_page=20)
-        return render_template(self.template, form=ActionForm(), pager=pager)
+        return super(PayoutListing, self).dispatch_request(
+                division_id, page, form=ActionForm())
 
 
 def register_perm_request_listing(app, endpoint, path, permissions, statuses):
@@ -259,28 +260,6 @@ class RequestForm(Form):
             raise ValidationError([str(e) for e in failures])
 
 
-def submit_divisions(user):
-    """Get a list of the divisions the given user is able to submit requests
-    to.
-
-    :param user: The user to evaluate.
-    :type user: :py:class:`~.models.User`
-    :returns: A list of tuples. The tuples are in the form (division.id,
-        division.name)
-    :rtype: list
-    """
-    submit_perms = user.permissions\
-            .filter_by(permission=PermissionType.submit)\
-            .subquery()
-    divisions = db.session.query(Division).join(submit_perms)\
-            .order_by(Division.name)
-    # Remove duplicates and sort divisions by name
-    choices = []
-    for name, group in groupby(divisions, lambda d: d.name):
-        choices.append((next(group).id, name))
-    return choices
-
-
 @blueprint.route('/add/', methods=['GET', 'POST'])
 @login_required
 def submit_request():
@@ -296,7 +275,8 @@ def submit_request():
         abort(403)
     form = RequestForm()
     # Create a list of divisions this user can submit to
-    form.division.choices = submit_divisions(current_user)
+    form.division.choices = current_user.submit_divisions()
+
     if form.validate_on_submit():
         mail = form.killmail
         # Prevent submitting other people's killmails
@@ -316,11 +296,11 @@ def submit_request():
             srp_request.pilot = pilot
             db.session.add(srp_request)
             db.session.commit()
-            return redirect(url_for('.request_detail',
+            return redirect(url_for('.get_request_details',
                 request_id=srp_request.id))
         else:
             flash("This kill has already been submitted", 'warning')
-            return redirect(url_for('.request_detail',
+            return redirect(url_for('.get_request_details',
                 request_id=srp_request.id))
     return render_template('form.html', form=form)
 
@@ -357,141 +337,203 @@ class ActionForm(Form):
             validators=[AnyOf(ActionType.values())])
 
 
-@blueprint.route('/<int:request_id>/', methods=['GET', 'POST'])
-@login_required
-def request_detail(request_id):
-    """Renders the detail page for a :py:class:`~.models.Request`\.
-
-    This function is currently used for `all` request detail views, including
-    the reviewers and payers as well as the submitting user. It also enforces a
-    the evaluation workflow, which can be seen in the diagram below. In
-    addition, the payout amount can only be changed (either directly or through
-    modifiers) while the request is in the 'evaluating' state.
-
-    :param int request_id: the ID of the request
-
-    """
-    def render_details():
-        # Different templates are used for different roles
-        if review_perm.can():
-            template = 'request_review.html'
-        elif pay_perm.can():
-            template = 'request_detail.html'
-        elif current_user == srp_request.submitter:
-            template = 'request_detail.html'
-        else:
-            abort(403)
-        return render_template(template, srp_request=srp_request,
-                modifier_form=ModifierForm(formdata=None),
-                payout_form=PayoutForm(formdata=None),
-                action_form=ActionForm(formdata=None),
-                void_form=VoidModifierForm(formdata=None))
-
-    srp_request = Request.query.get_or_404(request_id)
-    review_perm = ReviewRequestsPermission(srp_request)
-    pay_perm = PayoutRequestsPermission(srp_request)
-    if request.method == 'POST':
-        failed = False
-        if request.form['id_'] == 'modifier':
-            form = ModifierForm()
-        elif request.form['id_'] == 'payout':
-            if not review_perm.can():
-                flash("Only reviewers can change the base payout.", 'error')
-                return render_details()
-            form = PayoutForm()
-        elif request.form['id_'] == 'action':
-            form = ActionForm()
-        elif request.form['id_'] == 'void':
-            form = VoidModifierForm()
-        else:
-            abort(400)
-        if form.validate():
-            if form.id_.data == 'modifier':
-                if 'bonus' in form.type_.data:
-                    value = form.value.data
-                elif 'deduct' in form.type_.data:
-                    vaue = form.value.data * -1
-                if 'abs' in form.type_.data:
-                    ModClass = AbsoluteModifier
-                elif 'rel' in form.type_.data:
-                    ModClass = RelativeModifier
-                try:
-                    mod = ModClass(srp_request, current_user, form.note.data,
-                            value)
-                except ModifierError as e:
-                    flash(e, 'error')
-                    return render_details()
-                if form.type_.data == 'rel-bonus':
-                    mod.type_ = 'percentage'
-                    mod.value = form.value.data
-                elif form.type_.data == 'rel-deduct':
-                    mod.type_ = 'percentage'
-                    mod.value = form.value.data * -1
-                elif form.type_.data == 'abs-bonus':
-                    mod.type_ = 'absolute'
-                    mod.value = form.value.data
-                elif form.type_.data == 'abs-deduct':
-                    mod.type_ = 'absolute'
-                    mod.value = form.value.data * -1
-                db.session.add(mod)
-                db.session.commit()
-            elif form.id_.data == 'payout':
-                try:
-                    srp_request.base_payout = form.value.data
-                    db.session.commit()
-                except ModifierError as e:
-                    flash(e, 'error')
-                    return render_details()
-            elif form.id_.data == 'void':
-                modifier_id = int(form.modifier_id.data)
-                modifier = Modifier.query.get(modifier_id)
-                if modifier is None:
-                    flash("Invalid modifier ID {}.".format(modifier_id),
-                            'error')
-                    return render_details()
-                try:
-                    modifier.void(current_user)
-                    db.session.commit()
-                except ModifierError as e:
-                    flash(e, 'error')
-                    return render_details()
-            elif form.id_.data == 'action':
-                type_ = ActionType.from_string(form.type_.data)
-                try:
-                    Action(srp_request, current_user, form.note.data, type_)
-                    db.session.commit()
-                except ActionError as e:
-                    flash(e, 'error')
-                    return render_details()
-        else:
-            # TODO: Actual error handling, probably using flash()
-            print(form.errors)
-    # Different templates are used for different roles
-    return render_details()
-
-
-class DetailForm(Form):
+class ChangeDetailsForm(Form):
+    id_ = HiddenField(default='details')
     details = TextAreaField('Details', validators=[InputRequired()])
-    submit = SubmitField('Submit')
 
 
-@blueprint.route('/<int:request_id>/update/', methods=['GET', 'POST'])
+class AddNote(Form):
+    id_ = HiddenField(default='note')
+    note = TextAreaField('Add Note',
+            description=("If you have something like '#{Kill ID}', it will be "
+                         "linkified to the corresponding request "
+                         "(if it exists). For example, #1234567 would be "
+                         "linked to the request for the kill with ID "
+                         "1234567."),
+            validators=[InputRequired()])
+
+
+killmail_re = re.compile(r'#(\d+)')
+
+
+@blueprint.route('/<int:request_id>/', methods=['GET'])
 @login_required
-def request_detail_update(request_id):
-    srp_request = Request.query.get_or_404(request_id)
-    # Only the submitter can change the details, and only if it isn't finalized
-    if current_user != srp_request.submitter or srp_request.finalized:
+def get_request_details(request_id=None, srp_request=None):
+    """Handles responding to all of the :py:class:`~.models.Request` detail
+    functions.
+
+    The various modifier functions all depend on this function to create the
+    actual response content.
+    Only one of the arguments is required. The ``srp_request`` argument is a
+    conveniece to other functions calling this function that have already
+    retrieved the request.
+
+    :param int request_id: the ID of the request.
+    :param srp_request: the request.
+    :type srp_request: :py:class:`~.models.Request`
+    """
+    if srp_request is None:
+        srp_request = Request.query.get_or_404(request_id)
+    # Different templates are used for different roles
+    if current_user.has_permission(PermissionType.review,
+            srp_request.division):
+        template = 'request_review.html'
+    elif current_user.has_permission(PermissionType.pay, srp_request.division):
+        template = 'request_pay.html'
+    elif current_user == srp_request.submitter:
+        template = 'request_detail.html'
+    else:
         abort(403)
-    form = DetailForm()
-    if form.validate_on_submit():
-        archive_note = 'Old Details: ' + srp_request.details
+    if request.wants_json:
+        # dump the load to encode srp_request as json and then get a dictionary
+        # form of it. We need this to add a few bits of information to the
+        # standard request encoding
+        enc_request = json.loads(json.dumps(srp_request))
+        enc_request['actions'] = srp_request.actions
+        enc_request['modifiers'] = srp_request.modifiers
+        valid_actions = map(
+                lambda a: a.value,
+                srp_request.valid_actions(current_user))
+        enc_request['valid_actions'] = valid_actions
+        enc_request['current_user'] = current_user._get_current_object()
+        return jsonify(enc_request)
+    return render_template(template, srp_request=srp_request,
+            modifier_form=ModifierForm(formdata=None),
+            payout_form=PayoutForm(formdata=None),
+            action_form=ActionForm(formdata=None),
+            void_form=VoidModifierForm(formdata=None),
+            details_form=ChangeDetailsForm(formdata=None, obj=srp_request),
+            note_form=AddNote(formdata=None))
+
+
+def _add_modifier(srp_request):
+    form = ModifierForm()
+    if form.validate():
+        if 'bonus' in form.type_.data:
+            value = form.value.data
+        elif 'deduct' in form.type_.data:
+            value = form.value.data * -1
+        if 'abs' in form.type_.data:
+            ModClass = AbsoluteModifier
+            value *= 1000000
+        elif 'rel' in form.type_.data:
+            ModClass = RelativeModifier
+            value /= 100
+        try:
+            mod = ModClass(srp_request, current_user, form.note.data, value)
+            db.session.add(mod)
+            db.session.commit()
+        except ModifierError as e:
+            flash(e, 'error')
+    return get_request_details(srp_request=srp_request)
+
+
+def _change_payout(srp_request):
+    review_perm = ReviewRequestsPermission(srp_request)
+    form = PayoutForm()
+    if not review_perm.can():
+        flash("Only reviewers can change the base payout.", 'error')
+    elif form.validate():
+        try:
+            srp_request.base_payout = form.value.data * 1000000
+            db.session.commit()
+        except ModifierError as e:
+            flash(e, 'error')
+    return get_request_details(srp_request=srp_request)
+
+
+def _add_action(srp_request):
+    form = ActionForm()
+    if form.validate():
+        type_ = ActionType.from_string(form.type_.data)
+        try:
+            Action(srp_request, current_user, form.note.data, type_)
+            db.session.commit()
+        except ActionError as e:
+            flash(e, 'error')
+    return get_request_details(srp_request=srp_request)
+
+
+def _void_modifier(srp_request):
+    form = VoidModifierForm()
+    if form.validate():
+        modifier_id = int(form.modifier_id.data)
+        modifier = Modifier.query.get(modifier_id)
+        if modifier is None:
+            flash("Invalid modifier ID {}.".format(modifier_id),
+                    'error')
+        else:
+            try:
+                modifier.void(current_user)
+                db.session.commit()
+            except ModifierError as e:
+                flash(e, 'error')
+    return get_request_details(srp_request=srp_request)
+
+
+def _change_details(srp_request):
+    form = ChangeDetailsForm()
+    if current_user != srp_request.submitter:
+        flash("Only the submitter can change the request details.", 'error')
+    elif srp_request.finalized:
+        flash("Details con only be changed when the request is still pending.",
+                'error')
+    elif form.validate():
+        archive_note = "Old Details: " + srp_request.details
         archive_action = Action(srp_request, current_user, archive_note)
         archive_action.type_ = ActionType.evaluating
         srp_request.details = form.details.data
         db.session.commit()
-        return redirect(url_for('.request_detail', request_id=request_id))
-    form.details.data = srp_request.details
-    return render_template('form.html', form=form)
+    return get_request_details(srp_request=srp_request)
+
+
+def _add_note(srp_request):
+    form = AddNote()
+    if not current_user.has_permission(PermissionType.elevated):
+        flash("You do not have permission to add a note to a user.", 'error')
+    elif form.validate():
+        # Linkify killmail IDs
+        note_content = Markup.escape(form.note.data)
+        for match in killmail_re.findall(note_content):
+            kill_id = int(match)
+            check_request = db.session.query(Request.id).filter_by(id=kill_id)
+            if db.session.query(check_request.exists()):
+                link = '<a href="{url}">#{kill_id}</a>'.format(
+                        url=url_for('.request_detail', request_id=kill_id),
+                        kill_id=kill_id)
+                link = Markup(link)
+                note_content = note_content.replace('#' + match, link)
+        # Create the note
+        note = Note(srp_request.submitter, current_user, note_content)
+        db.session.commit()
+    return get_request_details(srp_request=srp_request)
+
+
+@blueprint.route('/<int:request_id>/', methods=['POST'])
+@login_required
+def modify_request(request_id):
+    """Handles POST requests that modify :py:class:`~.models.Request`\s.
+
+    Because of the numerous possible forms, this function bounces execution to
+    a more specific function base on the form's "id_" field.
+
+    :param int request_id: the ID of the request.
+    """
+    srp_request = Request.query.get_or_404(request_id)
+    if request.form['id_'] == 'modifier':
+        return _add_modifier(srp_request)
+    elif request.form['id_'] == 'payout':
+        return _change_payout(srp_request)
+    elif request.form['id_'] == 'action':
+        return _add_action(srp_request)
+    elif request.form['id_'] == 'void':
+        return _void_modifier(srp_request)
+    elif request.form['id_'] == 'details':
+        return _change_details(srp_request)
+    elif request.form['id_'] == 'note':
+        return _add_note(srp_request)
+    else:
+        return abort(400)
 
 
 class DivisionChange(Form):
@@ -508,8 +550,12 @@ def request_change_division(request_id):
             current_user != srp_request.submitter or \
             srp_request.finalized:
         abort(403)
+    division_choices = srp_request.submitter.submit_divisions()
+    if len(division_choices) < 2:
+        flash("No other divisions to move to.", 'info')
+        return redirect(url_for('.request_detail', request_id=request_id))
     form = DivisionChange()
-    form.division.choices = submit_divisions(srp_request.submitter)
+    form.division.choices = division_choices
     if form.validate_on_submit():
         new_division = Division.query.get(form.division.data)
         archive_note = "Moving from division '{}' to division '{}'.".format(
@@ -528,54 +574,3 @@ def request_change_division(request_id):
             return redirect(url_for('.list_pending_requests'))
     form.division.data = srp_request.division.id
     return render_template('form.html', form=form)
-
-
-@blueprint.route('/notes/<int:user_id>/', methods=['GET', 'POST'])
-@login_required
-def user_notes(user_id):
-    if not current_user.has_permission(PermissionType.elevated):
-        abort(403)
-    user = User.query.get_or_404(user_id)
-    return render_template('notes.html', user=user)
-
-
-class AddNote(Form):
-    note = TextAreaField('Note',
-            description=("If you have something like '#{Kill ID}', it will be "
-                         "linkified to the corresponding request "
-                         "(if it exists). For example, #1234567 would be "
-                         "linked to the request for the kill with ID "
-                         "1234567."),
-            validators=[InputRequired()])
-    submit = SubmitField('Submit')
-
-
-killmail_re = re.compile(r'#(\d+)')
-
-
-@blueprint.route('/notes/<int:user_id>/add/', methods=['GET', 'POST'])
-@login_required
-def add_user_note(user_id):
-    if not current_user.has_permission(PermissionType.elevated):
-        abort(403)
-    user = User.query.get_or_404(user_id)
-    form = AddNote()
-    if form.validate_on_submit():
-        # Linkify killmail IDs
-        note_content = Markup.escape(form.note.data)
-        for match in killmail_re.findall(note_content):
-            kill_id = int(match)
-            srp_request = db.session.query(Request.id).filter_by(id=kill_id)
-            print(kill_id, srp_request.all())
-            if db.session.query(srp_request.exists()):
-                link = '<a href="{url}">#{kill_id}</a>'.format(
-                        url=url_for('.request_detail', request_id=kill_id),
-                        kill_id=kill_id)
-                link = Markup(link)
-                note_content = note_content.replace('#' + match, link)
-        # Create the note
-        note = Note(user, current_user, note_content)
-        db.session.add(note)
-        db.session.commit()
-        return redirect(url_for('.user_notes', user_id=user_id))
-    return render_template('form.html', form=form)
